/*
 * Aquesta app està subjecta a una llicència de
 * Reconeixement-NoComercial-
 * SenseObraDerivada 3.0 Espanya de Creative
 * Common
 */

package controller;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.StringRequest;

import org.json.JSONException;
import org.json.JSONObject;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import edu.uoc.emsobraunaentrada.LoginActivity;
import edu.uoc.emsobraunaentrada.MainActivity;
import edu.uoc.emsobraunaentrada.R;
import model.Publicacio;
import model.Usuari;

/**
 * Created by Cesc on 21/05/2016.
 */
public class Controller extends Application {
    private Usuari usr;
    private ArrayList<Publicacio> publicacions = new ArrayList<>();
    private Publicacio pubSeleccionada;

    public DbConnection dbConnection = new DbConnection();
    public String nextActivity;

    // Mètodes per a les publicacions
    public void addPublicacions(List<Publicacio> publicacios) {
        this.publicacions.addAll(publicacios);
    }
    public void clearPublicacions() {
        publicacions.clear();
    }
    public boolean hasPubSelected() {
        return (pubSeleccionada != null);
    }
    public Publicacio getPubSeleccionada() {
        return pubSeleccionada;
    }
    public String getTitolPub() {
        return pubSeleccionada.getTitol();
    }
    public Publicacio getPublicacio(int pPosition) {

        return publicacions.get(pPosition);
    }
    public void setPubSeleccionada(int pPosition) {
        pubSeleccionada = publicacions.get(pPosition);
    }
    public void setPubPublished (int idPublicacio, String titol, String descripcio, int imgCategoria, String preu,
                                 String categoria, String localitat, String entrades, String dataEsdeve) {
        int intEntrades = Integer.parseInt(entrades);
        Date dtPub = new Date();
        Date dtEsdeve = getStringToDate("yyyy-MM-dd HH:mm:ss", dataEsdeve);

        pubSeleccionada = new Publicacio(idPublicacio, titol, descripcio, imgCategoria, preu,
                categoria, localitat, intEntrades, dtEsdeve, dtPub, usr);
    }
    public void clearPusSel() {
        pubSeleccionada = null;
    }

    // Mètodes per als usuaris
    public void setUsrSession (String nom, String mail, String localitat, String clauApi) {
        usr = new Usuari(nom, mail, localitat, clauApi);
    }
    public boolean hasUsrPublicacio() {
        return (pubSeleccionada.getUsrPublicacio() != null);
    }
    public boolean isLoggedUser () {
        return !(usr == null);
    }
    public Usuari getUsrRegistrat() {
        return this.usr;
    }
    public boolean setLogout() {
        try {
            usr = null;
            return true;
        }
        catch (Exception e) {
            return false;
        }
    }

    // Control errors de BBDD
    public String getDBError (Integer id) {
        String msg = getString(R.string.err_db);

        switch (id) {
            case 3:
                msg = getString(R.string.err_dbClauNoValida);
                break;
            case 5:
                msg = getString(R.string.err_dbDefault);
                break;
            case 6:
                msg = getString(R.string.err_dbParametres);
                break;
            case 7:
                msg = getString(R.string.err_dbUsrNoValid);
                break;
            case 8:
                msg = getString(R.string.err_dbUsrRegistrat);
                break;
            case 10:
                msg = getString(R.string.err_dbGetPubs);
                break;
            default:
                msg = getString(R.string.err_db);
                break;
        }
        return msg;


    }


    // Validacions genèriques
    public boolean isNumeric(String str)
    {
        try
        {
            double d = Double.parseDouble(str);
        }
        catch(NumberFormatException nfe)
        {
            return false;
        }
        return true;
    }

    public String getDateToDB (String inDate) {
        String dbDate = "";
        SimpleDateFormat df = new SimpleDateFormat("dd/MM/yyyy", Locale.US);
        Date convertedDate = new Date();
        try {
            convertedDate = df.parse(inDate);
            dbDate = getDateToDB(convertedDate);
        } catch (ParseException e) {
            e.printStackTrace();
            dbDate = "";
        }
        return dbDate;
    }

    public String getDateToDB (Date d) {
        String sdata = "";
        try {
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.US);
            sdata = df.format(d);
        }
        catch (Exception e) {
            sdata = "";
        }
        return sdata;
    }

    private Date getStringToDate(String inFormat, String sDate) {
        SimpleDateFormat df = new SimpleDateFormat(inFormat, Locale.US);
        Date convertedDate = new Date();
        try {
            convertedDate = df.parse(sDate);
        }
        catch (ParseException pe) {
            pe.printStackTrace();
        }
        return convertedDate;
    }
}
