/*
 * Aquesta app està subjecta a una llicència de
 * Reconeixement-NoComercial-
 * SenseObraDerivada 3.0 Espanya de Creative
 * Common
 */

package controller;

import model.Publicacio;
import ui.GetDataListener;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Application;
import android.net.Uri;
import android.os.AsyncTask;



/**
 * Created by Cesc on 11/05/2016.
 */
public class GetData extends AsyncTask<String, Void, String> {
    private final GetDataListener listener;
    private String msg;

    public GetData(GetDataListener listener) {
        this.listener = listener;
    }

    @Override
    protected String doInBackground(String... params) {
        // Obtener los parámetros de busqueda
        String categoria = params[0];
        String cadenaABuscar = params[1];
        String url = Uri.parse(AppConfig.URL_PUBLICACIO)
                .buildUpon().appendQueryParameter("titol", cadenaABuscar)
                .appendQueryParameter("nomCategoria", categoria).build()
                .toString();

        try {
            // Creem connexió http
            URL myurl = new URL(url);

            // Ens connectem
            HttpURLConnection urlConnection = (HttpURLConnection) myurl.openConnection();
            urlConnection.setRequestMethod("GET");
            urlConnection.connect();

            int responseCode = urlConnection.getResponseCode(); //can call this instead of con.connect()

            // Obtenim la resposta
            InputStream is = urlConnection.getInputStream();
            if (is != null) {
                StringBuilder sb = new StringBuilder();
                String line;

                BufferedReader reader = new BufferedReader(new InputStreamReader(is));
                while ((line = reader.readLine()) != null) {
                    sb.append(line);
                }
                reader.close();
                is.close();

                return sb.toString();
            }
            else {
                msg = "No s'ha obtingut cap resposta";
                return null;
            }

        } catch (IOException e) {
            msg = "Problemes amb la connexión";
        }
        return null;
    }

    @Override
    protected void onPostExecute(String sJson) {
        if (sJson == null) {
            if (listener != null)
                listener.onFetchFailure(msg);
            return;
        }
        try {
            // Convertir la cadena json en un array json
            JSONObject jsonObject = new JSONObject(sJson);
            JSONArray aJson = jsonObject.getJSONArray("datos");

            // Crear la llista de publicacions
            List<Publicacio> pubs = new ArrayList<Publicacio>();
            String dateStr, mailUsr;

            for (int i = 0; i < aJson.length(); i++) {
                JSONObject json = aJson.getJSONObject(i);
                Publicacio pub = new Publicacio();
                // id_publicacio
                pub.setId_publicacio(Integer.parseInt(json.getString("idPublicacio")));
                // titol
                pub.setTitol(new String(json.getString("titol").getBytes(), "UTF-8"));
                // localitat
                pub.setLocalitat(new String(json.getString("localitat").getBytes(), "UTF-8"));
                // num entrades
                pub.setEntrades(Integer.parseInt(json.getString("entrades")));
                // data esdeveniment
                dateStr = json.getString("data");
                Date dataEsdeve = getStringToDate(dateStr);
                pub.setDataEsdeve(dataEsdeve);
                // descripcio
                pub.setDescripcio(new String(json.getString("descripcio").getBytes(), "UTF-8"));
                // preu
                pub.setPreu(json.getString("preu"));
                // categoria
                pub.setCategoria(json.getString("nomCategoria"));
                // data publicació
                dateStr = json.getString("dataPublicacio");
                Date dataPublicacio = getStringToDate(dateStr);
                pub.setDataPublicacio(dataPublicacio);
                // img categoria
                pub.setImgCategoria(Integer.parseInt(json.getString("entrades")));

                // recuperem les dades de l'usuari que ho publica
                mailUsr = json.getString("mail");
                if (!mailUsr.equals("")) {
                    pub.setUsrPublicacio();
                    pub.getUsrPublicacio().setMail(mailUsr);
                    pub.getUsrPublicacio().setNom(json.getString("nom"));
                    pub.getUsrPublicacio().setLocalitat(json.getString("localitatUsr"));
                }

                // Afegim a la lista
                pubs.add(pub);
            }

            // Notificar al listener que ya están todos los datos
            if (listener != null) {
                listener.onFetchComplete(pubs);
            }
        }
            catch (JSONException e) {
            msg = "Resposta no vàlida";
            System.out.println("**** Error JSON" + e.getMessage());
            if (listener != null)
                listener.onFetchFailure(msg);
            return;
        } catch (UnsupportedEncodingException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /*
     * Esta función convierte la respuesta en una cadena
     *
     * @param es
     *            la string que obtuvimos como respuesta
     * @return devuelve una string json
     * @throws IOException
     */
    public String streamToString(final InputStream is) throws IOException {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is));
        StringBuilder sb = new StringBuilder();
        String line = null;

        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            throw e;
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                throw e;
            }
        }

        return sb.toString();
    }

    private Date getStringToDate (String strDate) {
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date dataEsdeve = sdf.parse(strDate);
            return dataEsdeve;
        }
        catch (Throwable e) {
            return null;
        }

    }
}
