/*
 * Aquesta app està subjecta a una llicència de
 * Reconeixement-NoComercial-
 * SenseObraDerivada 3.0 Espanya de Creative
 * Common
 */

package edu.uoc.emsobraunaentrada;

import android.app.Dialog;
import android.app.DialogFragment;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.support.v4.app.NavUtils;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.AuthFailureError;
import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

import controller.AppConfig;
import controller.Controller;

public class DetallActivity extends AppCompatActivity {
    private Boolean esPubUsuari; // Si la publicació és de l'usuari
    private ProgressDialog pDialog;
    private Controller aController;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        final Button btnContact;
        final Button btnEdit;

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_detall);

        //Get Global Controller Class object (see application tag in AndroidManifest.xml)
        aController = (Controller) getApplicationContext();

        // Construïm la toolbar amb els seus botons
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        assert getSupportActionBar() != null;
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);

        btnContact = (Button) findViewById(R.id.btnContact);
        btnEdit = (Button) findViewById(R.id.btnEdit);

        esPubUsuari = false;
        // recuperem els camps a informar
        ImageView imgCategoria = (ImageView) findViewById(R.id.imgPublicacio);
        TextView txtvwTitol = (TextView) findViewById(R.id.txtvw_PublicacioTitol);
        TextView txtvwLocEsdeve = (TextView) findViewById(R.id.txtvw_localitat);
        TextView txtvwDataEsdeve = (TextView) findViewById(R.id.txtvw_dataEsdeve);
        TextView txtvwEntrades = (TextView) findViewById(R.id.txtvw_numEntrades);
        TextView txtvwPreu = (TextView) findViewById(R.id.txtvw_preu);
        TextView txtvwDescripcio = (TextView) findViewById(R.id.txtvw_PubDesc);
        TextView txtvwNomUsuari = (TextView) findViewById(R.id.txtvw_nomUusari);
        TextView txtvwLocUsuari = (TextView) findViewById(R.id.txtvw_locUsuari);
        TextView txtvwDataPub = (TextView) findViewById(R.id.txtvw_dataPublicacio);

        // Mostrem totes les dades de la publicació seleccionada
        if (aController.hasPubSelected()) {
            imgCategoria.setImageResource(aController.getPubSeleccionada().getImgCategoria());
            txtvwTitol.setText(aController.getPubSeleccionada().getTitol());
            txtvwLocEsdeve.setText(aController.getPubSeleccionada().getLocalitat());
            txtvwDataEsdeve.setText(aController.getPubSeleccionada().getStrDataEsdeve());
            txtvwEntrades.setText(getString(R.string.str_entrades) + " " + aController.getPubSeleccionada().getEntrades());
            txtvwPreu.setText(aController.getPubSeleccionada().getPreu() + getString(R.string.str_simbolEuro));
            txtvwDescripcio.setText(aController.getPubSeleccionada().getDescripcio());
            txtvwDataPub.setText(aController.getPubSeleccionada().getStrDataPub());

            if (aController.hasUsrPublicacio()) {
                txtvwNomUsuari.setText(aController.getPubSeleccionada().getUsrPublicacio().getNom());
                txtvwLocUsuari.setText(aController.getPubSeleccionada().getUsrPublicacio().getLocalitat());

                if (aController.isLoggedUser()) {
                    String mailUsrLogin = aController.getUsrRegistrat().getMail();
                    String mailUsrPub = aController.getPubSeleccionada().getUsrPublicacio().getMail();
                    esPubUsuari = (mailUsrLogin.equals(mailUsrPub));
                }
            }
        }

        if (esPubUsuari) {
            btnEdit.setVisibility(View.VISIBLE);
            btnContact.setVisibility(View.GONE);
        }
        else {
            btnContact.setVisibility(View.VISIBLE);
            btnEdit.setVisibility(View.GONE);
        }

        btnEdit.setOnClickListener(new View.OnClickListener() {

            public void onClick(View view) {
                // Llencem l'activitat d'edicio
                Intent i = new Intent(getApplication().getApplicationContext(), EditActivity.class);
                startActivity(i);
            }
        });

        btnContact.setOnClickListener(new View.OnClickListener() {

            public void onClick(View view) {
                Intent i;
                if (aController.isLoggedUser()) {
                    // Llencem l'activitat de contactar
                    i = new Intent(getApplication().getApplicationContext(), CommentActivity.class);
                    startActivity(i);
                }
                else {
                    aController.nextActivity="comment";
                    // Llencem l'activitat de login
                    i = new Intent(getApplication().getApplicationContext(), LoginActivity.class);
                    startActivity(i);
                }
            }
        });
    }

    @Override
    public void onBackPressed()
    {
        super.onBackPressed();
        Intent i = new Intent(getApplication().getApplicationContext(), MainActivity.class);
        startActivity(i);
        finish();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.common_activity_menu, menu);

        // Habilitem botó eliminar publicació si és necessari
        if (esPubUsuari) {
            MenuItem item = menu.findItem(R.id.action_delPublish);
            item.setVisible(true);
        }

        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                NavUtils.navigateUpFromSameTask(this);
                finish();
                return true;
            case R.id.action_delPublish:
                deletePub();
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private void deletePub() {
        // Tag used to cancel the request
        String tag_string_req = "del_publish";
        final String TAG = EditActivity.class.getSimpleName();

        // Progress dialog
        pDialog = new ProgressDialog(this);
        pDialog.setCancelable(false);
        pDialog.setMessage(getString(R.string.dlg_esborrant));
        showDialog();

        /** json object parameter**/
        try {
            String url;
            url = AppConfig.URL_PUBLICACIO;
            url += "/" + aController.getPubSeleccionada().getId_publicacio();
            Log.e("Url ApiService:", url + "");
            JSONObject jsonObject = new JSONObject();
            JsonObjectRequest joReq = new JsonObjectRequest(Request.Method.DELETE,
                    url, jsonObject, new Response.Listener<JSONObject>() {

                @Override
                public void onResponse(JSONObject joResponse) {
                    String message;
                    Log.d(TAG, "Publicacio Response: " + joResponse.toString());
                    hideDialog();

                    try {
                        int estat = joResponse.getInt("estado");
                        // Check for error node in json
                        if (estat == 2) {
                            // Publish successfully removed

                            message = getString(R.string.msg_pubEliminada);
                            Toast.makeText(getApplicationContext(),
                                    message, Toast.LENGTH_LONG).show();

                            // Llancem l'activitat llista publicacions
                            Intent i = new Intent(DetallActivity.this,
                                    MainActivity.class);
                            startActivity(i);
                            finish();
                        } else {
                            // Error in del publish. Get the error message
                            String errorMsg = aController.getDBError(estat);
                            Toast.makeText(getApplicationContext(),
                                    errorMsg, Toast.LENGTH_LONG).show();
                        }
                    } catch (JSONException e) {
                        // JSON error
                        e.printStackTrace();
                        Toast.makeText(getApplicationContext(), "Json error: " + e.getMessage(), Toast.LENGTH_LONG).show();
                    }
                }
            }, new Response.ErrorListener() {

                @Override
                public void onErrorResponse(VolleyError error) {
                    Log.e(TAG, "Pub Error: " + error.getMessage());
                    Toast.makeText(getApplicationContext(),
                            error.getMessage(), Toast.LENGTH_LONG).show();
                    hideDialog();
                }
            }) {
                @Override
                public Map<String, String> getHeaders() throws AuthFailureError {
                    return createBasicAuthHeader(aController.getUsrRegistrat().getClauApi(), "");
                }

                Map<String, String> createBasicAuthHeader(String username, String password) {
                    Map<String, String> headerMap = new HashMap<>();
                    headerMap.put("Authorization", username);
                    return headerMap;
                }
            };

            // Adding request to request queue
            aController.dbConnection.getInstance().addToRequestQueue(getApplicationContext(), joReq, tag_string_req);
        } catch (Exception e) {
            hideDialog();
            Log.d(TAG, "Pub Response: " + e.getMessage());
            Toast.makeText(getApplicationContext(),
                    getString(R.string.err_publish), Toast.LENGTH_LONG).show();
        }
    }

    private void showDialog() {
        if (!pDialog.isShowing())
            pDialog.show();
    }

    private void hideDialog() {
        if (pDialog.isShowing())
            pDialog.dismiss();
    }
}
