/*
 * Aquesta app està subjecta a una llicència de
 * Reconeixement-NoComercial-
 * SenseObraDerivada 3.0 Espanya de Creative
 * Common
 */

package edu.uoc.emsobraunaentrada;

import android.app.ProgressDialog;
import android.content.Intent;
import android.support.v4.app.NavUtils;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.AuthFailureError;
import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import controller.AppConfig;
import controller.Controller;

public class EditActivity extends AppCompatActivity {
    private Spinner iCategoria;
    private EditText iTitol;
    private EditText iLloc;
    private EditText iData;
    private EditText iEntrades;
    private EditText iPreu;
    private EditText iDescripcio;
    private int intEntrades;
    private Date dtDataEsdeve;

    private static final String TAG = EditActivity.class.getSimpleName();
    private ProgressDialog pDialog;
    private Controller aController;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        final Button btnEdita;

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_edit);

        //Get Global Controller Class object (see application tag in AndroidManifest.xml)
        aController = (Controller) getApplicationContext();

        // Construïm la toolbar amb els seus botons
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        assert getSupportActionBar() != null;
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);

        // Progress dialog
        pDialog = new ProgressDialog(this);
        pDialog.setCancelable(false);

        iCategoria = (Spinner) findViewById(R.id.itxtCategoria);
        iTitol = (EditText) findViewById(R.id.itxtTitol);
        iLloc = (EditText) findViewById(R.id.itxtLloc);
        iData = (EditText) findViewById(R.id.itxtDataEsdeve);
        iEntrades = (EditText) findViewById(R.id.itxtEntrades);
        iPreu = (EditText) findViewById(R.id.itxtPreu);
        iDescripcio = (EditText) findViewById(R.id.itxtDescripcio);
        btnEdita = (Button) findViewById(R.id.btnEdita);

        if (aController.hasPubSelected()) {
            // Si estem editant afegim dades de la publicació
            showData(aController);
        }

        btnEdita.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                validateToPublish();
            }
        });
    }

    private void showData (Controller c) {
        String[] cats = getResources().getStringArray(R.array.categoria);
        iCategoria.setSelection(Arrays.asList(cats).indexOf(c.getPubSeleccionada().getCategoria()));
        iTitol.setText(c.getPubSeleccionada().getTitol());
        iLloc.setText(c.getPubSeleccionada().getLocalitat());
        iData.setText(new StringBuilder().append(c.getPubSeleccionada().getStrDataEsdeve()));
        iEntrades.setText(String.valueOf(c.getPubSeleccionada().getEntrades()));
        iPreu.setText(String.valueOf(c.getPubSeleccionada().getPreu()));
        iDescripcio.setText(c.getPubSeleccionada().getDescripcio());
    }

    private void showDialog() {
        if (!pDialog.isShowing())
            pDialog.show();
    }

    private void hideDialog() {
        if (pDialog.isShowing())
            pDialog.dismiss();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                NavUtils.navigateUpFromSameTask(this);
                finish();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public void onBackPressed()
    {
        super.onBackPressed();
        Intent i = new Intent(getApplication().getApplicationContext(), MainActivity.class);
        startActivity(i);
        finish();
    }

    private void validateToPublish() {
        String categoria = iCategoria.getSelectedItem().toString();
        String titol = iTitol.getText().toString().trim();
        String lloc = iLloc.getText().toString().trim();
        String data = iData.getText().toString().trim();
        String entrades = iEntrades.getText().toString();
        String preu = iPreu.getText().toString().replace(",", ".");
        String descripcio = iDescripcio.getText().toString();

        boolean cancel = false;
        View focusView = null;

        // Validació preu
        if (TextUtils.isEmpty(preu)) {
            iPreu.setError(getString(R.string.error_field_required));
            focusView = iPreu;
            cancel = true;
        } else if (!aController.isNumeric(preu)) {
            iPreu.setError(getString(R.string.error_invalid_format));
            focusView = iPreu;
            cancel = true;
        }

        // Validació entrades
        if (TextUtils.isEmpty(entrades)) {
            iEntrades.setError(getString(R.string.error_field_required));
            focusView = iEntrades;
            cancel = true;
        } else if (!aController.isNumeric(entrades)) {
            iEntrades.setError(getString(R.string.error_invalid_format));
            focusView = iEntrades;
            cancel = true;
        }

        // Validació data esdeveniment
        if (TextUtils.isEmpty(data)) {
            iData.setError(getString(R.string.error_field_required));
            focusView = iData;
            cancel = true;
        } else {
            data = aController.getDateToDB(data);
            if (data.equals("")) {
                iData.setError(getString(R.string.error_invalid_format));
                focusView = iData;
                cancel = true;
            }
        }

        // Validació localitat
        if (TextUtils.isEmpty(lloc)) {
            iLloc.setError(getString(R.string.error_field_required));
            focusView = iLloc;
            cancel = true;
        }

        // Validació títol
        if (TextUtils.isEmpty(titol)) {
            iTitol.setError(getString(R.string.error_field_required));
            focusView = iTitol;
            cancel = true;
        }

        //Validació categoria
        if (iCategoria.getSelectedItemPosition() == 0) {
            ((TextView)iCategoria.getChildAt(0)).setError(getString(R.string.error_field_required));
            iCategoria.setFocusable(true);
            iCategoria.setFocusableInTouchMode(true);
            focusView = iCategoria;
            cancel = true;
        }

        if (cancel) {
            // There was an error; don't attempt login and focus the first
            // form field with an error.
            focusView.requestFocus();
        } else {
            publish(categoria, titol, lloc, data, entrades, preu, descripcio);
        }
    }

    private void publish(final String categoria, final String titol, final String lloc, final String data,
                              final String entrades, final String preu, final String descripcio) {
        final boolean esEdicio = aController.hasPubSelected();

        // Tag used to cancel the request
        String tag_string_req = "req_publish";

        pDialog.setMessage(getString(R.string.dlg_publicant));
        showDialog();

        /** json object parameter**/
        try {
            String url;
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("titol", titol);
            jsonObject.put("localitat", lloc);
            jsonObject.put("data", data);
            jsonObject.put("entrades", entrades);
            jsonObject.put("preu", preu);
            jsonObject.put("descripcio", descripcio);
            jsonObject.put("mailUsuari", aController.getUsrRegistrat().getMail());
            jsonObject.put("nomCategoria", categoria);

            Log.e("jsonObject params", jsonObject.toString() + "");
            url = AppConfig.URL_PUBLICACIO;

            if (esEdicio) {
                url += "/" + aController.getPubSeleccionada().getId_publicacio();
            }

            JsonObjectRequest joReq = new JsonObjectRequest(esEdicio ? Request.Method.PUT : Request.Method.POST,
                    url, jsonObject, new Response.Listener<JSONObject>() {

                @Override
                public void onResponse(JSONObject joResponse) {
                    String message;
                    Log.d(TAG, "Publicacio Response: " + joResponse.toString());
                    hideDialog();

                    try {
                        int estat = joResponse.getInt("estado");
                        // Check for error node in json
                        if (estat == 2) {
                            // Publish successfully stored in MySQL

                            // Guardem info publicació
                            int idPublicacio;
                            if (esEdicio)
                                idPublicacio = aController.getPubSeleccionada().getId_publicacio();
                            else
                                idPublicacio = joResponse.getInt("id");
                            int imgCategoria = getImgCategoria(categoria);

                            aController.setPubPublished(idPublicacio, titol, descripcio, imgCategoria, preu,
                                    categoria, lloc, entrades, data);

                            if (esEdicio)
                                message = getString(R.string.msg_pubModificada);
                            else
                                message = getString(R.string.msg_pubCreada);

                            Toast.makeText(getApplicationContext(),
                                    message, Toast.LENGTH_LONG).show();

                            // Llancem l'activitat detall
                            Intent i = new Intent(EditActivity.this,
                                    DetallActivity.class);
                            startActivity(i);
                            finish();
                        } else {
                            // Error in login. Get the error message
                            String errorMsg = aController.getDBError(estat);
                            Toast.makeText(getApplicationContext(),
                                    errorMsg, Toast.LENGTH_LONG).show();
                        }
                    } catch (JSONException e) {
                        // JSON error
                        e.printStackTrace();
                        Toast.makeText(getApplicationContext(), "Json error: " + e.getMessage(), Toast.LENGTH_LONG).show();
                    }

                }
            }, new Response.ErrorListener() {

                @Override
                public void onErrorResponse(VolleyError error) {
                    Log.e(TAG, "Pub Error: " + error.getMessage());
                    Toast.makeText(getApplicationContext(),
                            error.getMessage(), Toast.LENGTH_LONG).show();
                    hideDialog();
                }
            }) {
                @Override
                public Map<String, String> getHeaders() throws AuthFailureError {
                    return createBasicAuthHeader(aController.getUsrRegistrat().getClauApi(), "");
                }

                Map<String, String> createBasicAuthHeader(String username, String password) {
                    Map<String, String> headerMap = new HashMap<>();

                    //String encodedCredentials = Base64.encodeToString(credentials.getBytes(), Base64.NO_WRAP);
                    headerMap.put("Authorization", username);

                    return headerMap;
                }
            };

            // Adding request to request queue
            aController.dbConnection.getInstance().addToRequestQueue(getApplicationContext(), joReq, tag_string_req);
        }
        catch (Exception e) {
            hideDialog();
            Log.d(TAG, "Pub Response: " + e.getMessage());
            Toast.makeText(getApplicationContext(),
                    getString(R.string.err_publish), Toast.LENGTH_LONG).show();
        }
    }

    private int getImgCategoria (String categoria) {
        switch (categoria) {
            case "Esports":
                return R.drawable.ic_esports;
            case "Concerts":
                return R.drawable.ic_concerts;
            case "Teatre i dança":
                return R.drawable.ic_teatre;
            case "Cinema":
                return R.drawable.ic_cinema;
            case "Altres":
                return R.drawable.ic_altres;
            default:
                return R.drawable.ic_launcher;
        }
    }
}
