/*
 * Aquesta app està subjecta a una llicència de
 * Reconeixement-NoComercial-
 * SenseObraDerivada 3.0 Espanya de Creative
 * Common
 */

package edu.uoc.emsobraunaentrada;

import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;

import org.json.JSONException;
import org.json.JSONObject;

import controller.AppConfig;
import controller.Controller;

public class LoginActivity extends AppCompatActivity {
    private TextView lblGotoRegister;
    private EditText inputEmail;
    private EditText inputPassword;


    private static final String TAG = RegistreActivity.class.getSimpleName();
    private ProgressDialog pDialog;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        Button btnLogin;

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);

        // Construïm la toolbar amb els seus botons
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        assert getSupportActionBar() != null;
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);

        // Progress dialog
        pDialog = new ProgressDialog(this);
        pDialog.setCancelable(false);

        // Set up the login form.
        inputEmail = (EditText) findViewById(R.id.txtEmail);
        inputPassword = (EditText) findViewById(R.id.txtPass);
        btnLogin = (Button) findViewById(R.id.btnLogin);

        // Reset errors
        inputEmail.setError(null);
        inputPassword.setError(null);

        btnLogin.setOnClickListener(new View.OnClickListener() {

            public void onClick(View view) {
                String email = inputEmail.getText().toString();
                String password = inputPassword.getText().toString();

                boolean cancel = false;
                View focusView = null;

                // Check for a valid password, if the user entered one.
                if (TextUtils.isEmpty(password)) {
                    inputPassword.setError(getString(R.string.error_field_required));
                    focusView = inputPassword;
                    cancel = true;
                }

                // Check for a valid email address.
                if (TextUtils.isEmpty(email)) {
                    inputEmail.setError(getString(R.string.error_field_required));
                    focusView = inputEmail;
                    cancel = true;
                } else if (!isEmailValid(email)) {
                    inputEmail.setError(getString(R.string.error_invalid_email));
                    focusView = inputEmail;
                    cancel = true;
                }

                if (cancel) {
                    // There was an error; don't attempt login and focus the first
                    // form field with an error.
                    focusView.requestFocus();
                } else {
                    checkLogin(email, password);
                }
            }
        });

        lblGotoRegister = (TextView) findViewById(R.id.link_to_register);
        lblGotoRegister.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent itemintent = new Intent(LoginActivity.this,
                        RegistreActivity.class);
                LoginActivity.this.startActivity(itemintent);
            }
        });
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.common_activity_menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private boolean isEmailValid(String email) {
        //TODO: Replace this with your own logic
        return email.contains("@");
    }

    private boolean isPasswordValid(String password) {
        //TODO: Replace this with your own logic
        return password.length() > 4;
    }

    private void showDialog() {
        if (!pDialog.isShowing())
            pDialog.show();
    }

    private void hideDialog() {
        if (pDialog.isShowing())
            pDialog.dismiss();
    }

    /**
     * function to verify login details in mysql db
     * */
    private void checkLogin(final String email, final String password) {
        //Get Global Controller Class object (see application tag in AndroidManifest.xml)
        final Controller aController = (Controller) getApplicationContext();

        // Tag used to cancel the request
        String tag_string_req = "req_login";

        pDialog.setMessage(getString(R.string.dlg_autenticant));
        showDialog();

        /** json object parameter**/
        try {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("mail", email);
            jsonObject.put("paraulaDePas", password);
            Log.e("jsonObject params", jsonObject.toString() + "");

            JsonObjectRequest joReq = new JsonObjectRequest(Request.Method.POST,
                    AppConfig.URL_LOGIN, jsonObject, new Response.Listener<JSONObject>() {

                @Override
                public void onResponse(JSONObject joResponse) {
                    Log.d(TAG, "Login Response: " + joResponse.toString());
                    hideDialog();

                    try {
                        int estat = joResponse.getInt("estado");
                        // Check for error node in json
                        if (estat == 2) {
                            // user successfully logged in

                            // Recuperem resposta JSON
                            JSONObject user = joResponse.getJSONObject("usuari");
                            String name = user.getString("nom");
                            String email = user.getString("mail");
                            String localitat = user.getString("localitat");
                            String clauApi = user.getString("clauApi");

                            // Guardem info usuari
                            aController.setUsrSession(name, email, localitat, clauApi);

                            Toast.makeText(getApplicationContext(),
                                    (getString(R.string.msg_holadenou) + " " + name + "!"), Toast.LENGTH_LONG).show();

                            // Llancem l'activitat corresponent
                            Intent i;
                            switch (aController.nextActivity) {
                                case "edit":
                                    // Nova publicació o edició
                                    i = new Intent(LoginActivity.this,
                                            EditActivity.class);
                                    startActivity(i);
                                    break;
                                case "comment":
                                    String usrMail = aController.getUsrRegistrat().getMail();
                                    String usrPubMail = aController.getPubSeleccionada().getUsrPublicacio().getMail();
                                    if (usrMail.equals(usrPubMail)) {
                                        // Activity edició si és usr publicacio
                                        i = new Intent(LoginActivity.this,
                                                EditActivity.class);
                                        startActivity(i);
                                    }
                                    else {
                                        // Activity comentari si no és usr publicacio
                                        i = new Intent(LoginActivity.this,
                                                CommentActivity.class);
                                        startActivity(i);
                                    }
                                    break;
                                default:
                                    break;
                            }
                            aController.nextActivity="";
                            finish();
                        } else {
                            // Error in login. Get the error message
                            String errorMsg = aController.getDBError(estat);
                            Toast.makeText(getApplicationContext(),
                                    errorMsg, Toast.LENGTH_LONG).show();
                        }
                    } catch (JSONException e) {
                        // JSON error
                        e.printStackTrace();
                        Toast.makeText(getApplicationContext(), "Json error: " + e.getMessage(), Toast.LENGTH_LONG).show();
                    }

                }
            }, new Response.ErrorListener() {

                @Override
                public void onErrorResponse(VolleyError error) {
                    Log.e(TAG, "Login Error: " + error.getMessage());
                    Toast.makeText(getApplicationContext(),
                            error.getMessage(), Toast.LENGTH_LONG).show();
                    hideDialog();
                }
            });

            // Adding request to request queue
            aController.dbConnection.getInstance().addToRequestQueue(getApplicationContext(), joReq, tag_string_req);
        }
        catch (Exception e) {
            hideDialog();
            Log.d(TAG, "Login Response: " + e.getMessage());
            Toast.makeText(getApplicationContext(),
                    getString(R.string.err_login), Toast.LENGTH_LONG).show();
        }
    }
}
