
/*
 * Aquesta app està subjecta a una llicència de
 * Reconeixement-NoComercial-
 * SenseObraDerivada 3.0 Espanya de Creative
 * Common
 */

package edu.uoc.emsobraunaentrada;

import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Build;
import android.support.design.widget.FloatingActionButton;
import android.support.v7.app.AppCompatActivity;
import android.support.v4.app.Fragment;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.app.SearchManager;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.SearchView;
import android.widget.SearchView.OnQueryTextListener;
import android.widget.Spinner;
import android.widget.Toast;

import java.util.List;

import controller.Controller;
import controller.GetData;
import controller.PubsAdaptador;
import model.Publicacio;
import ui.GetDataListener;

public class MainActivity extends AppCompatActivity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        // Afegim el Fragment que conté la listVeiw
        if (savedInstanceState == null) {
            getSupportFragmentManager().beginTransaction()
                    .add(R.id.container, new PlaceholderFragment()).commit();
        }
    }

    /**
     * A placeholder fragment containing a simple view.
     */
    public static class PlaceholderFragment extends Fragment implements
            GetDataListener {
        public PlaceholderFragment() {
        }

        ListView lvPublicacions;
        Spinner spCategories;
        SearchView svRapida;
        FloatingActionButton fab;
        Controller aController;

        @Override
        public View onCreateView(LayoutInflater inflater, ViewGroup container,
                                 Bundle savedInstanceState) {
            View rootView = inflater.inflate(R.layout.fragment_main, container, false);
            lvPublicacions = (ListView) rootView.findViewById(R.id.lv_publicacions);
            spCategories = (Spinner) rootView.findViewById(R.id.spin_categoria);
            fab = (FloatingActionButton) rootView.findViewById(R.id.pubsFAB);

            // recuperem la controller del context
            aController = (Controller) container.getContext().getApplicationContext();

            SearchManager searchManager = (SearchManager) getActivity()
                    .getSystemService(Context.SEARCH_SERVICE);
            svRapida = (SearchView) rootView.findViewById(R.id.svTitol);
            svRapida.setSearchableInfo(searchManager
                    .getSearchableInfo(getActivity().getComponentName()));
            svRapida.setIconifiedByDefault(true);

            inicializar();

            // Implementem filtrat per categoria
            spCategories
                    .setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                        @Override
                        public void onItemSelected(AdapterView<?> parent,
                                                   View view, int position, long id) {
                            Filtrar();
                        }

                        @Override
                        public void onNothingSelected(AdapterView<?> parent) {

                        }

                    });

            // Implementem filtrat per títol
            svRapida.setOnQueryTextListener(new OnQueryTextListener() {
                @Override
                public boolean onQueryTextSubmit(String query) {

                    Filtrar();
                    return false;
                }

                @Override
                public boolean onQueryTextChange(String newText) {
                    if (TextUtils.isEmpty(newText)) {
                        Filtrar();
                    }
                    return false;
                }

            });

            // Definim l'acció del FAB
            fab.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    aController.clearPusSel();
                    // Si l'usuari s'ha loguejat, mostrem pantalla nova publicació
                    // sino anem a la pantalla de login
                    if (aController.isLoggedUser()){
                        // Llancem l'activitat de nova i edició
                        Intent i = new Intent(getActivity().getApplicationContext(), EditActivity.class);
                        startActivity(i);
                    }
                    else {
                        // Llancem l'activitat de login
                        Intent i = new Intent(getActivity().getApplicationContext(), LoginActivity.class);
                        startActivity(i);
                        aController.nextActivity = "edit";
                    }
                }
            });
            return rootView;
        }

        public void inicializar() {
            // Fem la consulta inicial sense filtres
            GetData dades = new GetData(this);

            // Depenent de la versió executem la consulta en paralel
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB)
                dades.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, "", "");
            else
                dades.execute("", "");
        }

        public void Filtrar() {

            // Buidem les publicacions actuals
            aController.clearPublicacions();

            // Fem la consulta amb el títol i categoria seleccionats
            GetData dades = new GetData(this);
            String strFiltreCat = "";
            String FiltreTitol = svRapida.getQuery().toString();
            int FiltroCat = spCategories.getSelectedItemPosition();
            if (FiltroCat != 0) {
                strFiltreCat = spCategories.getSelectedItem().toString();
            }

            // Depenent de la versió executem la consulta en paralel
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB)
                dades.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR, strFiltreCat, FiltreTitol);
            else
                dades.execute(strFiltreCat, FiltreTitol);
        }

        @Override
        public void onFetchComplete(List<Publicacio> data) {
            // Afegim les publicacions a l'array de publicacions
            aController.addPublicacions(data);

            // Assignem l'adaptador a la listview
            lvPublicacions.setAdapter(new PubsAdaptador(getActivity(), data));

            // Implementem el click sobre una publicació
            lvPublicacions.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    // Desem la publicacio seleccionada a la controller
                    aController.setPubSeleccionada(position);

                    // Llencem l'activitat del detall d'una publicació
                    Intent i = new Intent(getActivity().getApplication().getApplicationContext(), DetallActivity.class);
                    startActivity(i);
                }
            });
        }

        @Override
        public void onFetchFailure(String msg) {
            // Mostrar que es lo que falló
            Toast.makeText(getActivity(), msg, Toast.LENGTH_LONG).show();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.main_activity_menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        //Get Global Controller Class object (see application tag in AndroidManifest.xml)
        final Controller aController = (Controller) getApplicationContext();

        Intent i;
        switch (item.getItemId()) {
            case R.id.action_about:
                // Llancem l'activitat about_activity
                i = new Intent(getApplication().getApplicationContext(), AboutActivity.class);
                startActivity(i);
                return true;
            case R.id.action_login:
                if (aController.isLoggedUser()) {
                    action(getString(R.string.msg_usrLoguejat));
                }
                else {
                    aController.nextActivity = "";
                    // Llancem l'activitat de login
                    i = new Intent(getApplication().getApplicationContext(), LoginActivity.class);
                    startActivity(i);
                }
                return true;
            case R.id.action_registre:
                if (aController.isLoggedUser()) {
                    action(getString(R.string.msg_usrRegistrat));
                }
                else {
                    aController.nextActivity = "";
                    // Llencem l'activitat de Registre
                    i = new Intent(getApplication().getApplicationContext(), RegistreActivity.class);
                    startActivity(i);
                }
                return true;
            case R.id.action_logout:
                if (aController.isLoggedUser()) {
                    String nomUsr = aController.getUsrRegistrat().getNom();
                    if (aController.setLogout())
                        action(getString(R.string.msg_logoutOk) + " " + nomUsr + "!");
                    else
                        action(getString(R.string.msg_logoutKo) +  "!");
                }
                else
                    action(getString(R.string.msg_usrNoLoguejat));

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private void action(String message) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
    }
}
