/*
 * Aquesta app està subjecta a una llicència de
 * Reconeixement-NoComercial-
 * SenseObraDerivada 3.0 Espanya de Creative
 * Common
 */

package edu.uoc.emsobraunaentrada;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;

import org.json.JSONException;
import org.json.JSONObject;

import controller.AppConfig;
import controller.Controller;

/**
 * Created by Cesc on 15/05/2016.
 */
public class RegistreActivity extends AppCompatActivity {
    private TextView lblGotoLogin;
    private Button btnRegister;
    private EditText inputName;
    private EditText inputEmail;
    private EditText inputLocalitat;
    private EditText inputPassword;
    private TextView registerErrorMsg;

    private static final String TAG = RegistreActivity.class.getSimpleName();
    private ProgressDialog pDialog;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_registre);

        // Progress dialog
        pDialog = new ProgressDialog(this);
        pDialog.setCancelable(false);

        // Construïm la toolbar amb els seus botons
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        assert getSupportActionBar() != null;
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);

        inputName = (EditText) findViewById(R.id.txtUserName);
        inputEmail = (EditText) findViewById(R.id.txtEmail);
        inputLocalitat = (EditText) findViewById(R.id.txtLocalitat);
        inputPassword = (EditText) findViewById(R.id.txtPass);
        btnRegister = (Button) findViewById(R.id.btnRegister);

        registerErrorMsg = (TextView) findViewById(R.id.register_error);

        btnRegister.setOnClickListener(new View.OnClickListener() {

            public void onClick(View view) {
                String name = inputName.getText().toString().trim();
                String email = inputEmail.getText().toString().trim();
                String localitat = inputLocalitat.getText().toString().trim();
                String password = inputPassword.getText().toString().trim();

                boolean cancel = false;
                View focusView = null;

                // Check for a valid password, if the user entered one.
                if (TextUtils.isEmpty(password)) {
                    inputPassword.setError(getString(R.string.error_field_required));
                    focusView = inputPassword;
                    cancel = true;
                } else if (!isPasswordValid(password)) {
                    inputPassword.setError(getString(R.string.error_invalid_password));
                    focusView = inputPassword;
                    cancel = true;
                }

                // Check for a valid email address.
                if (TextUtils.isEmpty(email)) {
                    inputEmail.setError(getString(R.string.error_field_required));
                    focusView = inputEmail;
                    cancel = true;
                } else if (!isEmailValid(email)) {
                    inputEmail.setError(getString(R.string.error_invalid_email));
                    focusView = inputEmail;
                    cancel = true;
                }

                if (cancel) {
                    // There was an error; don't attempt login and focus the first
                    // form field with an error.
                    //focusView.requestFocus();
                    marcarFocus(focusView);
                } else {
                    registerUser(name, email, localitat, password);
                }
            }
        });


        lblGotoLogin = (TextView) findViewById(R.id.link_to_login);
        lblGotoLogin.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {Intent itemintent = new Intent(RegistreActivity.this, LoginActivity.class);
                RegistreActivity.this.startActivity(itemintent);}
        });

    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.common_activity_menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private boolean isEmailValid(String email) {
        //TODO: Replace this with your own logic
        return email.contains("@");
    }

    private boolean isPasswordValid(String password) {
        //TODO: Replace this with your own logic
        return password.length() > 4;
    }

    private void showDialog() {
        if (!pDialog.isShowing())
            pDialog.show();
    }

    private void hideDialog() {
        if (pDialog.isShowing())
            pDialog.dismiss();
    }

    /**
     * function to verify login details in mysql db
     * */
    private void registerUser(final String name, final String email, final String localitat, final String password) {
        //Get Global Controller Class object (see application tag in AndroidManifest.xml)
        final Controller aController = (Controller) getApplicationContext();

        // Tag used to cancel the request
        String tag_string_req = "req_regiser";

        pDialog.setMessage(getString(R.string.dlg_registrant));
        showDialog();

        /** json object parameter**/
        try {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("nom", name);
            jsonObject.put("mail", email);
            jsonObject.put("localitat", localitat);
            jsonObject.put("paraulaDePas", password);
            Log.e("jsonObject params", jsonObject.toString() + "");

            JsonObjectRequest joReq = new JsonObjectRequest(Request.Method.POST,
                    AppConfig.URL_REGISTER, jsonObject, new Response.Listener<JSONObject>() {

                @Override
                public void onResponse(JSONObject joResponse) {
                    Log.d(TAG, "Register Response: " + joResponse.toString());
                    hideDialog();

                    try {
                        int estat = joResponse.getInt("estado");
                        // Check for error node in json
                        if (estat == 2) {
                            // User successfully stored in MySQL

                            // Now store the user in SQLite
                            String clauApi = joResponse.getString("clauApi");

                            // Guardem info usuari
                            aController.setUsrSession(name, email, localitat, clauApi);

                            // Inserting row in users table
                            //db.addUser(name, email, localitat, clauApi);

                            Toast.makeText(getApplicationContext(),
                                    (getString(R.string.msg_benvingut) + " " + name + "!"), Toast.LENGTH_LONG).show();

                            // Llancem l'activitat corresponent
                            Intent i;
                            switch (aController.nextActivity) {
                                case "edit":
                                    // Nova publicació o edició
                                    i = new Intent(RegistreActivity.this,
                                            EditActivity.class);
                                    startActivity(i);
                                    break;
                                case "comment":
                                    // Enviar comentari
                                    i = new Intent(RegistreActivity.this,
                                            CommentActivity.class);
                                    startActivity(i);
                                    break;
                                default:
                                    break;
                            }
                            aController.nextActivity="";
                            finish();
                        } else {
                            // Error in login. Get the error message
                            String errorMsg = aController.getDBError(estat);

                            if (estat == 8) {   // Usuari ja registrat
                                inputEmail.setError(errorMsg);
                                marcarFocus(inputEmail);
                            }
                            else
                                Toast.makeText(getApplicationContext(),
                                        errorMsg, Toast.LENGTH_LONG).show();
                        }
                    } catch (JSONException e) {
                        // JSON error
                        e.printStackTrace();
                        Toast.makeText(getApplicationContext(), "Json error: " + e.getMessage(), Toast.LENGTH_LONG).show();
                    }

                }
            }, new Response.ErrorListener() {

                @Override
                public void onErrorResponse(VolleyError error) {
                    Log.e(TAG, "Register Error: " + error.getMessage());
                    Toast.makeText(getApplicationContext(),
                            error.getMessage(), Toast.LENGTH_LONG).show();
                    hideDialog();
                }
            });

            // Adding request to request queue
            aController.dbConnection.getInstance().addToRequestQueue(getApplicationContext(), joReq, tag_string_req);
        }
        catch (Exception e) {
            hideDialog();
            Log.d(TAG, "Register Response: " + e.getMessage());
            Toast.makeText(getApplicationContext(),
                    getString(R.string.err_registre), Toast.LENGTH_LONG).show();
        }
    }

    private void marcarFocus (View focusView) {
        focusView.requestFocus();
    }
}
