<?php
	class publicaciones
	{
		const ESTADO_EXITO = 2;
		const ESTADO_NO_ENCONTRADO = 4;
		const ESTADO_ERROR_BD = 5;
		const ESTADO_ERROR_PARAMETROS = 6;
		const ESTADO_ERROR = 10; // Error recuperant publicacions
		
		// Dades de la taula 'publicacio'
		const TABLENAME = "publicacio";
		const TABLEUSRNAME = "usuari";
		const TITLE = "titol";
		const PLACE = "localitat";
		const EVENTDATE = "data";
		const TICKETS = "entrades";
		const PRICE = "preu";
		const DESCRIPTION = "descripcio";
		const MAIL = "mailUsuari";
		const CATEGORY = "nomCategoria";
		const PUBDATE = "dataPublicacio";
		const PUBLISHID = "idPublicacio";

		public static function get($peticion)
		{      
			if (empty($peticion[0]))
				return self::obtenerPublicaciones();
			else
				return self::obtenerPublicaciones($peticion[0]);		
		}

		public static function post()
		{
			$mailUsuario = usuarios::autorizar();

			$body = file_get_contents('php://input');
			$publicacion = json_decode($body);

			$idPublicacio = self::crear($mailUsuario, $publicacion);

			http_response_code(200);
			return [
				"estado" => self::ESTADO_EXITO,
				"mensaje" => "Publicación creada",
				"id" => $idPublicacio
			];		
		}

		public static function put($peticion)
		{
			$mailUsuario = usuarios::autorizar();

			if (!empty($peticion[0])) {
				$body = file_get_contents('php://input');
				$publicacion = json_decode($body);

				if (self::actualizar($mailUsuario, $publicacion, $peticion[0]) > 0) {
					http_response_code(200);
					return [
						"estado" => self::ESTADO_EXITO,
						"mensaje" => "Registro actualizado correctamente"
					];
				} else {
					throw new ExcepcionApi(self::ESTADO_NO_ENCONTRADO,
						"La publicacion que intentas acceder no existe", 404);
				}
			} else {
				throw new ExcepcionApi(self::ESTADO_ERROR_PARAMETROS, "Falta id", 422);
			}		
		}

		public static function delete($peticion)
		{
			$mailUsuario = usuarios::autorizar();

			if (!empty($peticion[0])) {
				if (self::eliminar($mailUsuario, $peticion[0]) > 0) {
					http_response_code(200);
					return [
						"estado" => self::ESTADO_EXITO,
						"mensaje" => "Registro eliminado correctamente"
					];
				} else {
					throw new ExcepcionApi(self::ESTADO_NO_ENCONTRADO,
						"La publicación no existe" . $mailUsuario . ";" . $peticion[0], 404);
				}
			} else {
				throw new ExcepcionApi(self::ESTADO_ERROR_PARAMETROS, "Falta id", 422);
			}		
		}
	   
		private function obtenerPublicaciones($idContacto = NULL)
		{
			try {
				$whereClause = self::construirWhere();
				if (!$idContacto) {
					
					$comando = "SELECT " . self::TABLENAME . ".*, " . self::TABLEUSRNAME . ".nom, " .
					           self::TABLEUSRNAME . ".mail, " . self::TABLEUSRNAME . ".localitat AS localitatUsr FROM " . 
							   self::TABLENAME . " INNER JOIN " . self::TABLEUSRNAME . " ON " .
							   self::TABLENAME . ".mailUsuari=" . self::TABLEUSRNAME . ".mail " . $whereClause;

					// Preparar sentencia
					$sentencia = ConexionBD::obtenerInstancia()->obtenerBD()->prepare($comando);
				} else {
					$comando = "SELECT * FROM " . self::TABLENAME  . $whereClause;

					// Preparar sentencia
					$sentencia = ConexionBD::obtenerInstancia()->obtenerBD()->prepare($comando);
				}

				// Ejecutar sentencia preparada
				if ($sentencia->execute()) {
					http_response_code(200);
					return
						[
							"estado" => self::ESTADO_EXITO,
							"datos" => $sentencia->fetchAll(PDO::FETCH_ASSOC)
						];
				} else
					throw new ExcepcionApi(self::ESTADO_ERROR, "Se ha producido un error");

			} catch (PDOException $e) {
				throw new ExcepcionApi(self::ESTADO_ERROR_BD, $e->getMessage());
			}
		}

		private function crear($mailUsuario, $publicacion)
		{
			if ($publicacion) {
				try {

					$pdo = ConexionBD::obtenerInstancia()->obtenerBD();

					// Sentencia INSERT
					$comando = "INSERT INTO " . self::TABLENAME . " ( " .
						self::TITLE . "," .
						self::PLACE . "," .
						self::EVENTDATE . "," .
						self::TICKETS . "," .
						self::PRICE . "," .
						self::DESCRIPTION . "," .
						self::MAIL . "," .
						self::CATEGORY . "," .
						self::PUBDATE . ")" .
						" VALUES(?,?,?,?,?,?,?,?,?)";

					// Preparar la sentencia
					$sentencia = $pdo->prepare($comando);

					$sentencia->bindParam(1, $titol);
					$sentencia->bindParam(2, $localitat);
					$sentencia->bindParam(3, $data);
					$sentencia->bindParam(4, $entrades);
					$sentencia->bindParam(5, $preu);
					$sentencia->bindParam(6, $descripcio);
					$sentencia->bindParam(7, $mailUsuari);
					$sentencia->bindParam(8, $nomCategoria);
					$sentencia->bindParam(9, $dataPublicacio);


					$titol = $publicacion->titol;
					$localitat = $publicacion->localitat;
					$data = $publicacion->data;
					$entrades = $publicacion->entrades;
					$preu = $publicacion->preu;
					$descripcio = $publicacion->descripcio;
					$mailUsuari = $publicacion->mailUsuari;
					$nomCategoria = $publicacion->nomCategoria;
					$dataPublicacio = date('Y-m-d');

					$sentencia->execute();

					// Retornar el último id insertado
					return $pdo->lastInsertId();

				} catch (PDOException $e) {
					throw new ExcepcionApi(self::ESTADO_ERROR_BD, $e->getMessage());
				}
			} else {
				throw new ExcepcionApi(
					self::ESTADO_ERROR_PARAMETROS, 
					utf8_encode("Error en existencia o sintaxis de parámetros"));
			}

		}	

		private function actualizar($mailUsuario, $publicacion, $idPublicacion)
		{
			try {
				// Creando consulta UPDATE
				$consulta = "UPDATE " . self::TABLENAME .
					" SET " . self::TITLE . "=?," .
					self::PLACE . "=?," .
					self::EVENTDATE . "=?," .
					self::TICKETS . "=?," .
					self::PRICE . "=?," .
					self::DESCRIPTION . "=?," .
					self::CATEGORY . "=? " .
					" WHERE " . self::PUBLISHID . "=? AND " . self::MAIL . "=?";

				// Preparar la sentencia
				$sentencia = ConexionBD::obtenerInstancia()->obtenerBD()->prepare($consulta);

				$sentencia->bindParam(1, $titol);
				$sentencia->bindParam(2, $localitat);
				$sentencia->bindParam(3, $data);
				$sentencia->bindParam(4, $entrades);
				$sentencia->bindParam(5, $preu);
				$sentencia->bindParam(6, $descripcio);
				$sentencia->bindParam(7, $nomCategoria);
				$sentencia->bindParam(8, $idPublicacion);
				$sentencia->bindParam(9, $mailUsuario);

				$titol = $publicacion->titol;
				$localitat = $publicacion->localitat;
				$data = $publicacion->data;
				$entrades = $publicacion->entrades;
				$preu = $publicacion->preu;
				$descripcio = $publicacion->descripcio;
				$nomCategoria = $publicacion->nomCategoria;

				// Ejecutar la sentencia
				$sentencia->execute();

				return $sentencia->rowCount();

			} catch (PDOException $e) {
				throw new ExcepcionApi(self::ESTADO_ERROR_BD, $e->getMessage());
			}
		}

		private function eliminar($mailUsuario, $idPublicacion)
		{
			try {
				// Sentencia DELETE
				$comando = "DELETE FROM " . self::TABLENAME .
					" WHERE " . self::PUBLISHID . "=? AND " .
					self::MAIL . "=?";

				// Preparar la sentencia
				$sentencia = ConexionBD::obtenerInstancia()->obtenerBD()->prepare($comando);

				$sentencia->bindParam(1, $idPublicacion);
				$sentencia->bindParam(2, $mailUsuario);

				$sentencia->execute();

				return $sentencia->rowCount();

			} catch (PDOException $e) {
				throw new ExcepcionApi(self::ESTADO_ERROR_BD, $e->getMessage());
			}
		}

		private function construirWhere() {
			$filtresDisponibles = array('nomCategoria' => 'string', 'titol' => 'stringLike');
			$whereClause = " WHERE " . self::EVENTDATE . " >= CURDATE() ";
			
			// recorrem els filtres disponibles per si algún ve com a paràmetre
			foreach ($filtresDisponibles as $filtre => $tipus) {
				// Recuperem els valors dels paràmetres per muntar el where
				$valorFiltre = (string)filter_input(INPUT_GET, $filtre);
				if ($valorFiltre) {
					if ($whereClause == "") {
						$whereClause = " WHERE ";
					}
					else {
						$whereClause = $whereClause . " AND ";
					}
					
					// Preparem el where en funció del tipus de dada (string, date, int, ...)
					switch ($tipus) {
						case 'string':
							$whereClause = $whereClause . $filtre . "='" . $valorFiltre . "'";
							break;
						case 'stringLike':
							$whereClause = $whereClause . "LOWER(". $filtre . ")" . " Like LOWER('%" . $valorFiltre . "%')";
							break;
						default:
							$whereClause = $whereClause . $filtre . "='" . $valorFiltre . "'";
							break;
					}					
				}
			}
			return $whereClause;
		}
	}
?>