<?php
	require_once "utilities/dbconnection.php";
	require_once "utilities/exceptionApi.php";
	
	class usuarios
	{
		const ESTADO_EXITO = 2;
		const ESTADO_CLAVE_NO_AUTORIZADA = 3;
		const ESTADO_AUSENCIA_CLAVE_API = 3;		
		const ESTADO_ERROR_BD = 5;
		const ESTADO_DATOS_ICORRECTOS = 7;
		const ESTADO_USUARIO_EXISTENTE = 8;
		const ESTADO_CREACION_FALLIDA = 9;
		const ESTADO_URL_INCORRECTA = 11;
		const ESTADO_FALLA_DESCONOCIDA	= 12;
		
		// Dades de la taula 'usuari'
		const TABLENAME = "usuari";
		const NAME = "nom";
		const MAIL = "mail";
		const PLACE = "localitat";
		const PASSWORD = "paraulaDePas";
		const API_KEY = "clauApi";

		public static function post($peticion)
		{
			// Procesar post
			if ($peticion[0] == 'registro') {
				return self::registrar();
			} else if ($peticion[0] == 'login') {
				return self::loguear();
			} else {
				throw new ExcepcionApi(self::ESTADO_URL_INCORRECTA, "Url mal formada", 400);
			}			
		}

		private function registrar()
		{
			$cuerpo = file_get_contents('php://input');
			$usuario = json_decode($cuerpo);
			
			// Generem clau api per retornar-la si creació correcte
			$clauApi = self::generarClaveApi();

			// Validar campos
			
			// Crear usuario
			$resultado = self::crear($usuario, $clauApi);
			
			switch ($resultado) {
				case self::ESTADO_EXITO:
					http_response_code(200);
					return
						[
							"estado" => self::ESTADO_EXITO,
							"clauApi" => $clauApi,
							"mensaje" => utf8_encode("¡Registro con éxito!")
						];
					break;
				case self::ESTADO_CREACION_FALLIDA:
					throw new ExcepcionApi(self::ESTADO_CREACION_FALLIDA, "Ha ocurrido un error");
					break;
				default:
					throw new ExcepcionApi(self::ESTADO_FALLA_DESCONOCIDA, "Falla desconocida", 400);
			}			
			
			// Imprimir respuesta
		}
		
		private function loguear()
		{
			$respuesta = array();

			$body = file_get_contents('php://input');
			$usuario = json_decode($body);

			$correo = $usuario->mail;
			$contrasena = $usuario->paraulaDePas;


			// Auteniquem, obtenim dades usuari i imprimim resposta
			if (self::autenticar($correo, $contrasena)) {
				$usuarioBD = self::obtenerUsuarioPorCorreo($correo);

				if ($usuarioBD != NULL) {
					http_response_code(200);
					$respuesta["nom"] = $usuarioBD["nom"];
					$respuesta["mail"] = $usuarioBD["mail"];
					$respuesta["localitat"] = $usuarioBD["localitat"];
					$respuesta["clauApi"] = $usuarioBD["clauApi"];
					return ["estado" => 2, "usuari" => $respuesta];
				} else {
					throw new ExcepcionApi(self::ESTADO_FALLA_DESCONOCIDA,
						"Error al validar l'usuari");
				}
			} else {
				throw new ExcepcionApi(self::ESTADO_DATOS_ICORRECTOS,
					utf8_encode("Mail o paraula de pas no vàlids"));
			}
		}

		private function obtenerUsuarioPorCorreo($correo)
		{
			$comando = "SELECT " .
				self::NAME . "," .
				self::MAIL . "," .
				self::PLACE . "," .
				self::PASSWORD . "," .
				self::API_KEY .
				" FROM " . self::TABLENAME .
				" WHERE " . self::MAIL . "=?";

			$sentencia = ConexionBD::obtenerInstancia()->obtenerBD()->prepare($comando);

			$sentencia->bindParam(1, $correo);

			if ($sentencia->execute())
				return $sentencia->fetch(PDO::FETCH_ASSOC);
			else
				return null;
		}		

		private function crear($datosUsuario, $clauApi)
		{
			$nom = $datosUsuario->nom;

			$paraulaDePas = $datosUsuario->paraulaDePas;
			$contrasenaEncriptada = self::encriptarContrasena($paraulaDePas);

			$mail = $datosUsuario->mail;
			$localitat = $datosUsuario->localitat;

			//$clauApi = self::generarClaveApi();

			try {

				$pdo = ConexionBD::obtenerInstancia()->obtenerBD();

				// Sentencia INSERT
				$comando = "INSERT INTO " . self::TABLENAME . " ( " .
					self::NAME . "," .
					self::MAIL . "," .
					self::PLACE . "," .
					self::PASSWORD . "," .
					self::API_KEY . ")" .
					" VALUES(?,?,?,?,?)";

				$sentencia = $pdo->prepare($comando);

				$sentencia->bindParam(1, $nom);
				$sentencia->bindParam(2, $mail);
				$sentencia->bindParam(3, $localitat);
				$sentencia->bindParam(4, $contrasenaEncriptada);
				$sentencia->bindParam(5, $clauApi);

				$resultado = $sentencia->execute();

				if ($resultado) {
					return self::ESTADO_EXITO;
				} else {
					return self::ESTADO_CREACION_FALLIDA;
				}
			} catch (PDOException $e) {
				if ($e->getCode() == 23000)	// Integrity error -> Usuari ja registrat
					throw new ExcepcionApi(self::ESTADO_USUARIO_EXISTENTE, $e->getMessage());
				else
					throw new ExcepcionApi(self::ESTADO_ERROR_BD, $e->getMessage());
			}
		}		
		
		private function encriptarContrasena($contrasenaPlana)
		{
			if ($contrasenaPlana)
				return password_hash($contrasenaPlana, PASSWORD_DEFAULT);
			else return null;
		}

		private function generarClaveApi()
		{
			return md5(microtime().rand());
		}

		private function autenticar($correo, $contrasena)
		{
			$comando = "SELECT paraulaDePas FROM " . self::TABLENAME .
				" WHERE " . self::MAIL . "=?";

			try {

				$sentencia = ConexionBD::obtenerInstancia()->obtenerBD()->prepare($comando);

				$sentencia->bindParam(1, $correo);

				$sentencia->execute();

				if ($sentencia) {
					$resultado = $sentencia->fetch();

					if (self::validarContrasena($contrasena, $resultado['paraulaDePas'])) {
						return true;
					} else return false;
				} else {
					return false;
				}
			} catch (PDOException $e) {
				throw new ExcepcionApi(self::ESTADO_ERROR_BD, $e->getMessage());
			}
		}

		private function validarContrasena($contrasenaPlana, $contrasenaHash)
		{
			return password_verify($contrasenaPlana, $contrasenaHash);
		}

		public static function autorizar()
		{
			$cabeceras = apache_request_headers();

			if (isset($cabeceras["Authorization"])) {

				$claveApi = $cabeceras["Authorization"];

				if (self::validarClaveApi($claveApi)) {
					return self::obtenerMailUsuario($claveApi);
				} else {
					throw new ExcepcionApi(
						self::ESTADO_CLAVE_NO_AUTORIZADA, "Clave de API no autorizada", 401);
				}

			} else {
				throw new ExcepcionApi(
					self::ESTADO_AUSENCIA_CLAVE_API,
					utf8_encode("Se requiere Clave del API para autenticación"));
					//"Cabeceras: " . $cabeceras["Content-Type"]);
			}
		}

		private function validarClaveApi($claveApi)
		{
			$comando = "SELECT COUNT(" . self::MAIL . ")" .
				" FROM " . self::TABLENAME .
				" WHERE " . self::API_KEY . "=?";

			$sentencia = ConexionBD::obtenerInstancia()->obtenerBD()->prepare($comando);

			$sentencia->bindParam(1, $claveApi);

			$sentencia->execute();

			return $sentencia->fetchColumn(0) > 0;
		}

		private function obtenerMailUsuario($claveApi)
		{
			$comando = "SELECT " . self::MAIL .
				" FROM " . self::TABLENAME .
				" WHERE " . self::API_KEY . "=?";

			$sentencia = ConexionBD::obtenerInstancia()->obtenerBD()->prepare($comando);

			$sentencia->bindParam(1, $claveApi);

			if ($sentencia->execute()) {
				$resultado = $sentencia->fetch();
				return $resultado['mail'];
			} else
				return null;
		}		
	}
?>